# Copyright

from __future__ import absolute_import

import stepper as _stepper
from .digital_port import DigitalPort as _DigitalPort


class Stepper(_stepper.Stepper):
    """Extend `stepper.Stepper` for easy configuration via `h5config`.

    Uses `DigitalPort` for transmitting the output.

    >>> from pycomedi import constant as _constant
    >>> from pyafm.config import StepperConfig, DigitalPortConfig

    >>> devices = []

    >>> config = StepperConfig()
    >>> config['port'] = DigitalPortConfig()
    >>> config['port']['channels'] = [1, 2, 3, 4]
    >>> config['port']['direction'] = _constant.IO_DIRECTION.output
    >>> config['port']['name'] = 'stepper port'
    >>> config['name'] = 'test stepper'

    >>> s = Stepper(config=config)
    >>> s.load_from_config(devices=devices)
    >>> s.position
    0
    >>> s.single_step(1)
    >>> s.position
    2

    >>> for device in devices:
    ...     device.close()
    """
    def __init__(self, config):
        self.config = config
        self.port = None
        c = self.config  # reduce verbosity
        super(Stepper, self).__init__(
            write=self.port.write_bitfield, full_step=c['full-step'],
            logic=c['logic'], delay=c['delay'], step_size=c['step-size'],
            backlash=c['backlash'])

    def load_from_config(self, devices):
        c = self.config  # reduce verbosity
        if self.port is None:
            self.port = _DigitalPort(config=c['port'])
            self.port.load_from_config(devices=devices)
            self._write = self.port.write_bitfield
        self.full_step = c['full-step']
        self.logic = c['logic']
        self.delay = c['delay']
        self.step_size = c['step-size']
        self.backlash = c['backlash']

    def setup_config(self):
        self.port.setup_config()
        self.config['port'] = self.port.config
        self.config['full-step'] = self.full_step
        self.config['logic'] = self.logic
        self.config['delay'] = self.delay
        self.config['step-size'] = self.step_size
        self.config['backlash'] = self.backlash
